@@ -120,6 +120,12 @@ static void gconf_handle_segv (int signum);
 
 static gboolean in_shutdown = FALSE;
 
+/*
+ * Flag indicating we received a SIGHUP and we should reaload
+ * all sources during the next periodic_cleanup()
+ */
+static gboolean need_db_reload = FALSE;
+
 /* 
  * CORBA goo
  */
@@ -439,7 +445,6 @@ signal_handler (int signo)
     break;
 
   case SIGTERM:
-  case SIGHUP:
     enter_shutdown ();
 
     /* let the fatal signals interrupt us */
@@ -452,6 +457,13 @@ signal_handler (int signo)
       gconf_main_quit ();
     break;
 
+  case SIGHUP:
+    --in_fatal;
+
+    /* reload sources during next periodic_cleanup() */
+    need_db_reload = TRUE;
+    break;
+
   case SIGUSR1:
     --in_fatal;
     
@@ -637,17 +649,17 @@ main(int argc, char** argv)
   act.sa_handler = signal_handler;
   act.sa_mask    = empty_mask;
   act.sa_flags   = 0;
-  sigaction (SIGTERM,  &act, 0);
-  sigaction (SIGILL,  &act, 0);
-  sigaction (SIGBUS,  &act, 0);
-  sigaction (SIGFPE,  &act, 0);
-  sigaction (SIGHUP,  &act, 0);
-  sigaction (SIGSEGV, &act, 0);
-  sigaction (SIGABRT, &act, 0);
-  sigaction (SIGUSR1,  &act, 0);
+  sigaction (SIGTERM,  &act, NULL);
+  sigaction (SIGILL,  &act, NULL);
+  sigaction (SIGBUS,  &act, NULL);
+  sigaction (SIGFPE,  &act, NULL);
+  sigaction (SIGHUP,  &act, NULL);
+  sigaction (SIGSEGV, &act, NULL);
+  sigaction (SIGABRT, &act, NULL);
+  sigaction (SIGUSR1,  &act, NULL);
   
   act.sa_handler = SIG_IGN;
-  sigaction (SIGINT, &act, 0);
+  sigaction (SIGINT, &act, NULL);
 
   CORBA_exception_init(&ev);
 
@@ -714,7 +726,7 @@ main(int argc, char** argv)
       char buf[1] = { 'g' };
       if (write (write_byte_fd, buf, 1) != 1)
         {
-          gconf_log (GCL_ERR, _("Failed to write byte to pipe fd %d so client program may hang: %s"), write_byte_fd, g_strerror (errno));
+          gconf_log (GCL_ERR, _("Failed to write byte to pipe file descriptor %d so client program may hang: %s"), write_byte_fd, g_strerror (errno));
         }
 
       close (write_byte_fd);
@@ -789,6 +801,16 @@ static gboolean need_log_cleanup = FALSE;
 static gboolean
 periodic_cleanup_timeout(gpointer data)
 {  
+  if (need_db_reload)
+    {
+      gconf_log (GCL_INFO, _("SIGHUP received, reloading all databases"));
+
+      need_db_reload = FALSE;
+      shutdown_databases ();
+      init_databases ();
+      gconf_server_load_sources ();		  
+    }
+  
   gconf_log (GCL_DEBUG, "Performing periodic cleanup, expiring cache cruft");
   
   drop_old_clients ();
@@ -1052,10 +1074,6 @@ shutdown_databases (void)
     g_hash_table_destroy(dbs_by_addresses);
 
   dbs_by_addresses = NULL;
-
-  if (default_db)
-    gconf_database_free (default_db);
-
   default_db = NULL;
 }
 
